@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2006 to 2013 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #ifndef NEED_GETOPT
 #include <getopt.h>
 #endif
@@ -67,6 +68,7 @@ static const char *help_text =
 "\t   Compile pronunciation rules and dictionary from the current\n"
 "\t   directory. <voice name> specifies the language\n"
 "--ipa      Write phonemes to stdout using International Phonetic Alphabet\n"
+"\t         --ipa=1 Use ties, --ipa=2 Use ZWJ, --ipa=3 Separate with _\n" 
 "--path=\"<path>\"\n"
 "\t   Specifies the directory containing the espeak-data directory\n"
 "--pho      Write mbrola phoneme data (.pho) to stdout or to the file in --phonout\n"
@@ -127,14 +129,16 @@ void DisplayVoices(FILE *f_out, char *language)
 	const char *p;
 	int len;
 	int count;
-	int scores = 0;
+	int c;
+	int j;
 	const espeak_VOICE *v;
 	const char *lang_name;
 	char age_buf[12];
+	char buf[80];
 	const espeak_VOICE **voices;
 	espeak_VOICE voice_select;
 
-	static char genders[4] = {' ','M','F',' '};
+	static char genders[4] = {'-','M','F','-'};
 
 	if((language != NULL) && (language[0] != 0))
 	{
@@ -144,14 +148,13 @@ void DisplayVoices(FILE *f_out, char *language)
 		voice_select.gender = 0;
 		voice_select.name = NULL;
 		voices = espeak_ListVoices(&voice_select);
-		scores = 1;
 	}
 	else
 	{
 		voices = espeak_ListVoices(NULL);
 	}
 
-	fprintf(f_out,"Pty Language Age/Gender VoiceName       File        Other Langs\n");
+	fprintf(f_out,"Pty Language Age/Gender VoiceName          File          Other Languages\n");
 
 	for(ix=0; (v = voices[ix]) != NULL; ix++)
 	{
@@ -169,8 +172,16 @@ void DisplayVoices(FILE *f_out, char *language)
 
 			if(count==0)
 			{
-				fprintf(f_out,"%2d  %-12s%s%c  %-17s %-11s ",
-               p[0],lang_name,age_buf,genders[v->gender],v->name,v->identifier);
+				for(j=0; j < sizeof(buf); j++)
+				{
+					// replace spaces in the name
+					if((c = v->name[j]) == ' ')
+						c = '_';
+					if((buf[j] = c) == 0)
+						break;
+				}
+				fprintf(f_out,"%2d  %-12s%s%c  %-20s %-13s ",
+               p[0],lang_name,age_buf,genders[v->gender],buf,v->identifier);
 			}
 			else
 			{
@@ -179,8 +190,6 @@ void DisplayVoices(FILE *f_out, char *language)
 			count++;
 			p += len+2;
 		}
-//		if(scores)
-//			fprintf(f_out,"%3d  ",v->score);
 		fputc('\n',f_out);
 	}
 }   //  end of DisplayVoices
@@ -213,14 +222,17 @@ int OpenWavFile(char *path, int rate)
 	if(path == NULL)
 		return(2);
 
-	if(path[0] == 0)
-		return(0);
-
-	if(strcmp(path,"stdout")==0)
-		f_wavfile = stdout;
-	else
-		f_wavfile = fopen(path,"wb");
+	while(isspace(*path)) path++;
 
+	f_wavfile = NULL;
+	if(path[0] != 0)
+	{
+		if(strcmp(path,"stdout")==0)
+			f_wavfile = stdout;
+		else
+			f_wavfile = fopen(path,"wb");
+	}
+	
 	if(f_wavfile == NULL)
 	{
 		fprintf(stderr,"Can't write to: '%s'\n",path);
@@ -317,6 +329,17 @@ static int SynthCallback(short *wav, int numsamples, espeak_EVENT *events)
 }
 
 
+static void PrintVersion()
+{//=======================
+	const char *version;
+	const char *path_data;
+	espeak_Initialize(AUDIO_OUTPUT_SYNCHRONOUS, 0, NULL, espeakINITIALIZE_DONT_EXIT);
+	version = espeak_Info(&path_data);
+	printf("eSpeak text-to-speech: %s  Data at: %s\n", version, path_data);
+}
+
+
+
 #ifdef NEED_GETOPT
 	struct option {
 		char *name;
@@ -355,7 +378,8 @@ int main (int argc, char **argv)
 		{"path",    required_argument, 0, 0x107},
 		{"phonout", required_argument, 0, 0x108},
 		{"pho",     no_argument,       0, 0x109},
-		{"ipa",     no_argument,       0, 0x10a},
+		{"ipa",     optional_argument, 0, 0x10a},
+		{"version", no_argument,       0, 0x10b},
 		{0, 0, 0, 0}
 		};
 
@@ -391,14 +415,10 @@ int main (int argc, char **argv)
 	espeak_VOICE voice_select;
 	char filename[200];
 	char voicename[40];
-	char voice_mbrola[20];
-	char dictname[40];
 #define N_PUNCTLIST  100
 	wchar_t option_punctlist[N_PUNCTLIST];
 
 	voicename[0] = 0;
-	voice_mbrola[0] = 0;
-	dictname[0] = 0;
 	wavefile[0] = 0;
 	filename[0] = 0;
 	option_punctlist[0] = 0;
@@ -481,15 +501,10 @@ int main (int argc, char **argv)
 			break;
 
 		case 'h':
-			{
-				const char *version;
-				const char *path_data;
-				espeak_Initialize(AUDIO_OUTPUT_SYNCHRONOUS, 0, data_path, espeakINITIALIZE_DONT_EXIT);
-				version = espeak_Info(&path_data);
-				printf("\n");
-				printf("eSpeak text-to-speech: %s  Data at: %s\n%s", version, path_data, help_text);
-				exit(0);
-			}
+			printf("\n");
+			PrintVersion();
+			printf("%s", help_text);
+			exit(0);
 			break;
 
 		case 'k':
@@ -605,8 +620,23 @@ int main (int argc, char **argv)
 
 		case 0x10a:  // --ipa
 			option_phonemes = 3;
+			if(optarg2 != NULL)
+			{
+				value = -1;
+				sscanf(optarg2,"%d",&value);
+				if((value<0) || (value>3))
+				{
+					fprintf(stderr,"Bad value for -ipa=\n");
+					value = 0;
+				}
+				option_phonemes += value;
+			}
 			break;
 
+		case 0x10b:  // -version
+			PrintVersion();
+			exit(0);
+
 		default:
 			exit(0);
 		}
@@ -765,7 +795,11 @@ int main (int argc, char **argv)
 		fclose(f_text);
 	}
 
-	espeak_Synchronize();
+	if(espeak_Synchronize() != EE_OK)
+	{
+		fprintf(stderr, "espeak_Synchronize() failed, maybe error when opening output device\n");
+		exit(4);
+	}
 
 	if(f_phonemes_out != stdout)
 		fclose(f_phonemes_out);  // needed for WinCE
