@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2010 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2013 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -40,7 +40,7 @@ static void SmoothSpect(void);
 
 // list of phonemes in a clause
 int n_phoneme_list=0;
-PHONEME_LIST phoneme_list[N_PHONEME_LIST];
+PHONEME_LIST phoneme_list[N_PHONEME_LIST+1];
 
 int mbrola_delay;
 char mbrola_name[20];
@@ -82,7 +82,7 @@ const char *WordToString(unsigned int word)
 	int  ix;
 	static char buf[5];
 
-	for(ix=0; ix<3; ix++)
+	for(ix=0; ix<4; ix++)
 		buf[ix] = word >> (ix*8);
 	buf[4] = 0;
 	return(buf);
@@ -483,7 +483,7 @@ static void set_frame_rms(frame_t *fr, int new_rms)
 		}
 		return;
 	}
- 
+
 	if(fr->rms == 0) return;    // check for divide by zero
 	x = (new_rms * 64)/fr->rms;
 	if(x >= 200) x = 199;
@@ -507,7 +507,7 @@ static void formants_reduce_hf(frame_t *fr, int level)
 
 	if(voice->klattv[0])
 		return;
- 
+
 	for(ix=2; ix < 8; ix++)
 	{
 		x = fr->fheight[ix] * level;
@@ -594,7 +594,7 @@ static void AdjustFormants(frame_t *fr, int target, int min, int max, int f1_adj
 		fr->ffreq[1] += x;
 		fr->ffreq[0] += x;
 	}
-	formants_reduce_hf(fr,hf_reduce); 
+	formants_reduce_hf(fr,hf_reduce);
 }
 
 
@@ -717,7 +717,7 @@ if(voice->klattv[0])
 				fr = CopyFrame(seq[n_frames-1].frame,0);
 				seq[n_frames-1].frame = fr;
 				rms = RMS_GLOTTAL1;
-	
+
 				// degree of glottal-stop effect depends on closeness of vowel (indicated by f1 freq)
 				modn_flags = 0x400 + (VowelCloseness(fr) << 8);
 			}
@@ -726,7 +726,7 @@ if(voice->klattv[0])
 				fr = DuplicateLastFrame(seq,n_frames++,len);
 				if(len > 36)
 					seq_len_adjust += (len - 36);
-	
+
 				if(f2 != 0)
 				{
 					AdjustFormants(fr, f2, f2_min, f2_max, f1, f3_adj, f3_amp, flags);
@@ -741,7 +741,7 @@ if(voice->klattv[0])
 				{
 					fr = CopyFrame(seq[ix].frame,0);
 					seq[ix].frame = fr;
-					
+
 					for(formant=1; formant<=5; formant++)
 					{
 						int x;
@@ -1182,7 +1182,7 @@ if(which==1)
 				q[1] = len + (modulation << 16);
 				q[2] = (long64)frame1;
 				q[3] = (long64)frame2;
-	
+
 				WcmdqInc();
 			}
 			last_frame = frame1 = frame2;
@@ -1348,6 +1348,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 	PHONEME_DATA phdata_next;
 	PHONEME_DATA phdata_tone;
 	FMT_PARAMS fmtp;
+	static WORD_PH_DATA worddata;
 
 	if(option_quiet)
 		return(0);
@@ -1372,10 +1373,11 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 		syllable_centre = -1;
 		last_pitch_cmd = -1;
 		memset(vowel_transition,0,sizeof(vowel_transition));
+		memset(&worddata, 0, sizeof(worddata));
 		DoPause(0,0);    // isolate from the previous clause
 	}
 
-	while(ix < (*n_ph))
+	while((ix < (*n_ph)) && (ix < N_PHONEME_LIST-2))
 	{
 		p = &phoneme_list[ix];
 
@@ -1424,7 +1426,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 
 		EndAmplitude();
 
-		if(p->prepause > 0)
+		if((p->prepause > 0) && !(p->ph->phflags & phPREVOICE))
 			DoPause(p->prepause,1);
 
 		done_phoneme_marker = 0;
@@ -1436,7 +1438,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			}
 			else
 			{
-				WritePhMnemonic(phoneme_name, p->ph, p, use_ipa);
+				WritePhMnemonic(phoneme_name, p->ph, p, use_ipa, NULL);
 				DoPhonemeMarker(espeakEVENT_PHONEME, sourceix, 0, phoneme_name);
 				done_phoneme_marker = 1;
 			}
@@ -1446,10 +1448,14 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 		{
 		case phPAUSE:
 			DoPause(p->length,0);
+#ifdef _ESPEAKEDIT
+            p->std_length = p->ph->std_length;
+#endif
 			break;
 
 		case phSTOP:
 			released = 0;
+			ph = p->ph;
 			if(next->type==phVOWEL)
 			{
 				 released = 1;
@@ -1463,13 +1469,30 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			if(released == 0)
 				p->synthflags |= SFLAG_NEXT_PAUSE;
 
-			InterpretPhoneme(NULL, 0, p, &phdata);
+			if(ph->phflags & phPREVOICE)
+			{
+				// a period of voicing before the release
+				memset(&fmtp, 0, sizeof(fmtp));
+				InterpretPhoneme(NULL, 0x01, p, &phdata, &worddata);
+				fmtp.fmt_addr = phdata.sound_addr[pd_FMT];
+				fmtp.fmt_amp = phdata.sound_param[pd_FMT];
+
+				if(last_pitch_cmd < 0)
+				{
+					DoAmplitude(next->amp,NULL);
+					DoPitch(envelope_data[p->env],next->pitch1,next->pitch2);
+				}
+
+				DoSpect2(ph, 0, &fmtp, p, 0);
+			}
+
+			InterpretPhoneme(NULL, 0, p, &phdata, &worddata);
 			phdata.pd_control |= pd_DONTLENGTHEN;
 			DoSample3(&phdata, 0, 0);
 			break;
 
 		case phFRICATIVE:
-			InterpretPhoneme(NULL, 0, p, &phdata);
+			InterpretPhoneme(NULL, 0, p, &phdata, &worddata);
 
 			if(p->synthflags & SFLAG_LENGTHEN)
 			{
@@ -1509,7 +1532,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			if((prev->type==phVOWEL) || (prev->ph->phflags & phVOWEL2) || (ph->phflags & phPREVOICE))
 			{
 				// a period of voicing before the release
-				InterpretPhoneme(NULL, 0x01, p, &phdata);
+				InterpretPhoneme(NULL, 0x01, p, &phdata, &worddata);
 				fmtp.fmt_addr = phdata.sound_addr[pd_FMT];
 				fmtp.fmt_amp = phdata.sound_param[pd_FMT];
 
@@ -1537,7 +1560,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			{
 				p->synthflags |= SFLAG_NEXT_PAUSE;
 			}
-			InterpretPhoneme(NULL,0, p, &phdata);
+			InterpretPhoneme(NULL,0, p, &phdata, &worddata);
 			fmtp.fmt_addr = phdata.sound_addr[pd_FMT];
 			fmtp.fmt_amp = phdata.sound_param[pd_FMT];
 			fmtp.wav_addr = phdata.sound_addr[pd_ADDWAV];
@@ -1582,7 +1605,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			{
 				p->synthflags |= SFLAG_NEXT_PAUSE;
 			}
-			InterpretPhoneme(NULL,0, p, &phdata);
+			InterpretPhoneme(NULL,0, p, &phdata, &worddata);
 			memset(&fmtp, 0, sizeof(fmtp));
 			fmtp.std_length = phdata.pd_param[i_SET_LENGTH]*2;
 			fmtp.fmt_addr = phdata.sound_addr[pd_FMT];
@@ -1608,7 +1631,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 				last_frame = NULL;
 			}
 
-			InterpretPhoneme(NULL,0, p, &phdata);
+			InterpretPhoneme(NULL,0, p, &phdata, &worddata);
 			fmtp.std_length = phdata.pd_param[i_SET_LENGTH]*2;
 			fmtp.fmt_addr = phdata.sound_addr[pd_FMT];
 			fmtp.fmt_amp = phdata.sound_param[pd_FMT];
@@ -1653,7 +1676,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			{
 				StartSyllable();
 			}
-			InterpretPhoneme(NULL, 0, p, &phdata);
+			InterpretPhoneme(NULL, 0, p, &phdata, &worddata);
 			fmtp.std_length = phdata.pd_param[i_SET_LENGTH]*2;
 			fmtp.fmt_addr = phdata.sound_addr[pd_FMT];
 			fmtp.fmt_amp = phdata.sound_param[pd_FMT];
@@ -1669,7 +1692,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 
 			memset(&fmtp, 0, sizeof(fmtp));
 
-			InterpretPhoneme(NULL, 0, p, &phdata);
+			InterpretPhoneme(NULL, 0, p, &phdata, &worddata);
 			fmtp.std_length = phdata.pd_param[i_SET_LENGTH] * 2;
 
 			if(((fmtp.fmt_addr = phdata.sound_addr[pd_VWLSTART]) != 0) && ((phdata.pd_control & pd_FORNEXTPH) == 0))
@@ -1681,7 +1704,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			if(prev->type != phPAUSE)
 			{
 				// check the previous phoneme
-				InterpretPhoneme(NULL, 0, prev, &phdata_prev);
+				InterpretPhoneme(NULL, 0, prev, &phdata_prev, NULL);
 				if((fmtp.fmt_addr = phdata_prev.sound_addr[pd_VWLSTART]) != 0)
 				{
 					// a vowel start has been specified by the Vowel program
@@ -1746,7 +1769,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 
 			if((option_phoneme_events) && (done_phoneme_marker == 0))
 			{
-				WritePhMnemonic(phoneme_name, p->ph, p, use_ipa);
+				WritePhMnemonic(phoneme_name, p->ph, p, use_ipa, NULL);
 				DoPhonemeMarker(espeakEVENT_PHONEME, sourceix, 0, phoneme_name);
 			}
 
@@ -1763,7 +1786,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			if(next->type != phPAUSE)
 			{
 				fmtp.fmt2_lenadj = 0;
-				InterpretPhoneme(NULL, 0, next, &phdata_next);
+				InterpretPhoneme(NULL, 0, next, &phdata_next, NULL);
 
 				fmtp.use_vowelin = 1;
 				fmtp.transition0 = phdata_next.vowel_transition[2];  // always do vowel_transition, even if ph_VWLEND ??  consider [N]
@@ -1864,7 +1887,6 @@ int SpeakNextClause(FILE *f_in, const void *text_in, int control)
 		n_phoneme_list = 0;
 		WcmdqStop();
 
-		embedded_value[EMBED_T] = 0;
 		return(0);
 	}
 
@@ -1931,20 +1953,14 @@ int SpeakNextClause(FILE *f_in, const void *text_in, int control)
 
 	if((option_phonemes > 0) || (phoneme_callback != NULL))
 	{
-		int use_ipa = 0;
-		if(option_phonemes == 3)
-			use_ipa = 1;
+		int phoneme_mode = 0;
+		if(option_phonemes >= 3)
+			phoneme_mode = 0x10 + option_phonemes-3;   // 0x10=ipa, 0x11=ipa with tie, 0x12=ipa with ZWJ, 0x13=ipa with separators
 
-		GetTranslatedPhonemeString(translator->phon_out, sizeof(translator->phon_out), use_ipa);
+		GetTranslatedPhonemeString(translator->phon_out, sizeof(translator->phon_out), phoneme_mode);
 		if(option_phonemes > 0)
 		{
 			fprintf(f_trans,"%s\n",translator->phon_out);
-	
-			if(!iswalpha(0x010d))
-			{
-				// check that c-caron is recognized as an alphabetic character
-				fprintf(stderr,"Warning: Accented letters are not recognized, eg: U+010D\nSet LC_CTYPE to a UTF-8 locale\n");
-			}
 		}
 		if(phoneme_callback != NULL)
 		{
