@@ -53,18 +53,161 @@ enum {ONE_BILLION=1000000000};
 #endif
 
 
+ 
+ 
+#ifdef USE_PULSEAUDIO
+// create some wrappers for runtime detection
+
+// checked on wave_init
+static int pulse_running;
+
+// wave.cpp (this file)
+int wave_port_init(int);
+void* wave_port_open(const char* the_api);
+size_t wave_port_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSize);
+int wave_port_close(void* theHandler);
+int wave_port_is_busy(void* theHandler);
+void wave_port_terminate();
+uint32_t wave_port_get_read_position(void* theHandler);
+uint32_t wave_port_get_write_position(void* theHandler);
+void wave_port_flush(void* theHandler);
+void wave_port_set_callback_is_output_enabled(t_wave_callback* cb);
+void* wave_port_test_get_write_buffer();
+int wave_port_get_remaining_time(uint32_t sample, uint32_t* time);
+
+// wave_pulse.cpp
+int is_pulse_running();
+int wave_pulse_init(int);
+void* wave_pulse_open(const char* the_api);
+size_t wave_pulse_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSize);
+int wave_pulse_close(void* theHandler);
+int wave_pulse_is_busy(void* theHandler);
+void wave_pulse_terminate();
+uint32_t wave_pulse_get_read_position(void* theHandler);
+uint32_t wave_pulse_get_write_position(void* theHandler);
+void wave_pulse_flush(void* theHandler);
+void wave_pulse_set_callback_is_output_enabled(t_wave_callback* cb);
+void* wave_pulse_test_get_write_buffer();
+int wave_pulse_get_remaining_time(uint32_t sample, uint32_t* time);
+
+// wrappers
+int wave_init(int srate) {
+  pulse_running = is_pulse_running();
+
+  if (pulse_running)
+    return wave_pulse_init(srate);
+  else
+    return wave_port_init(srate);
+}
+
+void* wave_open(const char* the_api) {
+  if (pulse_running)
+    return wave_pulse_open(the_api);
+  else
+    return wave_port_open(the_api);
+}
+
+size_t wave_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSize) {
+  if (pulse_running)
+    return wave_pulse_write(theHandler, theMono16BitsWaveBuffer, theSize);
+  else
+    return wave_port_write(theHandler, theMono16BitsWaveBuffer, theSize);
+}
+
+int wave_close(void* theHandler) {
+  if (pulse_running)
+    return wave_pulse_close(theHandler);
+  else
+    return wave_port_close(theHandler);
+}
+
+int wave_is_busy(void* theHandler) {
+  if (pulse_running)
+    return wave_pulse_is_busy(theHandler);
+  else
+    return wave_port_is_busy(theHandler);
+}
+
+void wave_terminate() {
+  if (pulse_running)
+    wave_pulse_terminate();
+  else
+    wave_port_terminate();
+}
+
+uint32_t wave_get_read_position(void* theHandler) {
+  if (pulse_running)
+    return wave_pulse_get_read_position(theHandler);
+  else
+    return wave_port_get_read_position(theHandler);
+}
+
+uint32_t wave_get_write_position(void* theHandler) {
+  if (pulse_running)
+    return wave_pulse_get_write_position(theHandler);
+  else
+    return wave_port_get_write_position(theHandler);
+}
+
+void wave_flush(void* theHandler) {
+  if (pulse_running)
+    wave_pulse_flush(theHandler);
+  else
+    wave_port_flush(theHandler);
+}
+
+void wave_set_callback_is_output_enabled(t_wave_callback* cb) {
+  if (pulse_running)
+    wave_pulse_set_callback_is_output_enabled(cb);
+  else
+    wave_port_set_callback_is_output_enabled(cb);
+}
+
+void* wave_test_get_write_buffer() {
+  if (pulse_running)
+    return wave_pulse_test_get_write_buffer();
+  else
+    return wave_port_test_get_write_buffer();
+}
+
+int wave_get_remaining_time(uint32_t sample, uint32_t* time)
+{
+  if (pulse_running)
+    return wave_pulse_get_remaining_time(sample, time);
+  else
+    return wave_port_get_remaining_time(sample, time);
+}
+
+// rename functions to be wrapped
+#define wave_init wave_port_init
+#define wave_open wave_port_open
+#define wave_write wave_port_write
+#define wave_close wave_port_close
+#define wave_is_busy wave_port_is_busy
+#define wave_terminate wave_port_terminate
+#define wave_get_read_position wave_port_get_read_position
+#define wave_get_write_position wave_port_get_write_position
+#define wave_flush wave_port_flush
+#define wave_set_callback_is_output_enabled wave_port_set_callback_is_output_enabled
+#define wave_test_get_write_buffer wave_port_test_get_write_buffer
+#define wave_get_remaining_time wave_port_get_remaining_time
+
+#endif  // USE_PULSEAUDIO
+
+
 static t_wave_callback* my_callback_is_output_enabled=NULL;
 
 #define N_WAV_BUF   10
-#define SAMPLE_RATE 22050
+#define MAX_SAMPLE_RATE 22050
 #define FRAMES_PER_BUFFER 512
-#define BUFFER_LENGTH (SAMPLE_RATE*2*sizeof(uint16_t))
-#define THRESHOLD (BUFFER_LENGTH/5)
+#define BUFFER_LENGTH (MAX_SAMPLE_RATE*2*sizeof(uint16_t))
+//#define THRESHOLD (BUFFER_LENGTH/5)
 static char myBuffer[BUFFER_LENGTH];
 static char* myRead=NULL; 
 static char* myWrite=NULL; 
 static int out_channels=1;
 static int my_stream_could_start=0;
+static int wave_samplerate;
 
 static int mInCallbackFinishedState = false;
 #if (USE_PORTAUDIO == 18)
@@ -168,89 +311,89 @@ static int pa_callback(void *inputBuffer, void *outputBuffer,
 			  PaStreamCallbackFlags flags, void *userData )
 #endif
 {
-  int aResult=0; // paContinue
-  char* aWrite = myWrite;
-  size_t n = out_channels*sizeof(uint16_t)*framesPerBuffer;
+	int aResult=0; // paContinue
+	char* aWrite = myWrite;
+	size_t n = out_channels*sizeof(uint16_t)*framesPerBuffer;
 
-  myReadPosition += framesPerBuffer;
-  SHOW("pa_callback > myReadPosition=%u, framesPerBuffer=%lu (n=0x%x) \n",(int)myReadPosition, framesPerBuffer, n);
+	myReadPosition += framesPerBuffer;
+	SHOW("pa_callback > myReadPosition=%u, framesPerBuffer=%lu (n=0x%x) \n",(int)myReadPosition, framesPerBuffer, n);
 
-  if (aWrite >= myRead)
-    {
-      if((size_t)(aWrite - myRead) >= n)
+	if (aWrite >= myRead)
 	{
-	  memcpy(outputBuffer, myRead, n);
-	  myRead += n;
+		if((size_t)(aWrite - myRead) >= n)
+		{
+			memcpy(outputBuffer, myRead, n);
+			myRead += n;
+		}
+		else
+		{
+			SHOW_TIME("pa_callback > underflow");
+			aResult=1; // paComplete;
+			mInCallbackFinishedState = true;
+			size_t aUsedMem=0;
+			aUsedMem = (size_t)(aWrite - myRead);
+			if (aUsedMem)
+			{
+				memcpy(outputBuffer, myRead, aUsedMem);
+			}
+			char* p = (char*)outputBuffer + aUsedMem;
+			memset(p, 0, n - aUsedMem); 
+			//	  myReadPosition += aUsedMem/(out_channels*sizeof(uint16_t));
+			myRead = aWrite;
+		}
 	}
-      else
+	else // myRead > aWrite
 	{
-	  SHOW_TIME("pa_callback > underflow");
-	  aResult=1; // paComplete;
-	  mInCallbackFinishedState = true;
-	  size_t aUsedMem=0;
-	  aUsedMem = (size_t)(aWrite - myRead);
-	  if (aUsedMem)
-	    {
-	      memcpy(outputBuffer, myRead, aUsedMem);
-	    }
-	  char* p = (char*)outputBuffer + aUsedMem;
-	  memset(p, 0, n - aUsedMem); 
-	  //	  myReadPosition += aUsedMem/(out_channels*sizeof(uint16_t));
-	  myRead = aWrite;
-	}
-    }
-  else // myRead > aWrite
-    {
-      if ((size_t)(myBuffer + BUFFER_LENGTH - myRead) >= n)
-	{
-	  memcpy(outputBuffer, myRead, n);
-	  myRead += n;
-	}
-      else if ((size_t)(aWrite + BUFFER_LENGTH - myRead) >= n)
-	{
-	  int aTopMem = myBuffer + BUFFER_LENGTH - myRead;
-	  if (aTopMem)
-	    {
-	      SHOW("pa_callback > myRead=0x%x, aTopMem=0x%x\n",(int)myRead, (int)aTopMem);
-	      memcpy(outputBuffer, myRead, aTopMem);
-	    }
-	  int aRest = n - aTopMem;
-	  if (aRest)
-	    {
-	      SHOW("pa_callback > myRead=0x%x, aRest=0x%x\n",(int)myRead, (int)aRest);
-	      char* p = (char*)outputBuffer + aTopMem;
-	      memcpy(p, myBuffer, aRest);
-	    }
-	  myRead = myBuffer + aRest;
-	}
-      else
-	{ 
-	  SHOW_TIME("pa_callback > underflow");
-	  aResult=1; // paComplete;
+		if ((size_t)(myBuffer + BUFFER_LENGTH - myRead) >= n)
+		{
+			memcpy(outputBuffer, myRead, n);
+			myRead += n;
+		}
+		else if ((size_t)(aWrite + BUFFER_LENGTH - myRead) >= n)
+		{
+			int aTopMem = myBuffer + BUFFER_LENGTH - myRead;
+			if (aTopMem)
+			{
+				SHOW("pa_callback > myRead=0x%x, aTopMem=0x%x\n",(int)myRead, (int)aTopMem);
+				memcpy(outputBuffer, myRead, aTopMem);
+			}
+			int aRest = n - aTopMem;
+			if (aRest)
+			{
+				SHOW("pa_callback > myRead=0x%x, aRest=0x%x\n",(int)myRead, (int)aRest);
+				char* p = (char*)outputBuffer + aTopMem;
+				memcpy(p, myBuffer, aRest);
+			}
+			myRead = myBuffer + aRest;
+		}
+		else
+		{ 
+			SHOW_TIME("pa_callback > underflow");
+			aResult=1; // paComplete;
 
-	  int aTopMem = myBuffer + BUFFER_LENGTH - myRead;
-	  if (aTopMem)
-	    {
-	      SHOW("pa_callback > myRead=0x%x, aTopMem=0x%x\n",(int)myRead, (int)aTopMem);
-	      memcpy(outputBuffer, myRead, aTopMem);
-	    }
-	  int aRest = aWrite - myBuffer;
-	  if (aRest)
-	    {
-	      SHOW("pa_callback > myRead=0x%x, aRest=0x%x\n",(int)myRead, (int)aRest);
-	      char* p = (char*)outputBuffer + aTopMem;
-	      memcpy(p, myBuffer, aRest);
-	    }
+			int aTopMem = myBuffer + BUFFER_LENGTH - myRead;
+			if (aTopMem)
+			{
+				SHOW("pa_callback > myRead=0x%x, aTopMem=0x%x\n",(int)myRead, (int)aTopMem);
+				memcpy(outputBuffer, myRead, aTopMem);
+			}
+			int aRest = aWrite - myBuffer;
+			if (aRest)
+			{
+				SHOW("pa_callback > myRead=0x%x, aRest=0x%x\n",(int)myRead, (int)aRest);
+				char* p = (char*)outputBuffer + aTopMem;
+				memcpy(p, myBuffer, aRest);
+			}
 
-	  size_t aUsedMem = aTopMem + aRest;
-	  char* p = (char*)outputBuffer + aUsedMem;
-	  memset(p, 0, n - aUsedMem); 
-	  //	  myReadPosition += aUsedMem/(out_channels*sizeof(uint16_t));
-	  myRead = aWrite;
+			size_t aUsedMem = aTopMem + aRest;
+			char* p = (char*)outputBuffer + aUsedMem;
+			memset(p, 0, n - aUsedMem); 
+			//	  myReadPosition += aUsedMem/(out_channels*sizeof(uint16_t));
+			myRead = aWrite;
+		}
 	}
-    }
 
-  SHOW("pa_callback > myRead=%x\n",(int)myRead);
+	SHOW("pa_callback > myRead=%x\n",(int)myRead);
 
 
   // #if USE_PORTAUDIO == 18
@@ -288,7 +431,7 @@ static int pa_callback(void *inputBuffer, void *outputBuffer,
 #endif
 
 
-  return(aResult);
+	return(aResult);
   //#endif
 
 }  //  end of WaveCallBack
@@ -335,7 +478,7 @@ static int wave_open_sound()
       out_channels = 1;
 
 #if USE_PORTAUDIO == 18
-      //      err = Pa_OpenDefaultStream(&pa_stream,0,1,paInt16,SAMPLE_RATE,FRAMES_PER_BUFFER,N_WAV_BUF,pa_callback,(void *)userdata);
+      //      err = Pa_OpenDefaultStream(&pa_stream,0,1,paInt16,wave_samplerate,FRAMES_PER_BUFFER,N_WAV_BUF,pa_callback,(void *)userdata);
 
    PaDeviceID playbackDevice = Pa_GetDefaultOutputDeviceID();
 
@@ -351,7 +494,7 @@ static int wave_open_sound()
 				paInt16,
 				NULL,
 				/* general parameters */
-				SAMPLE_RATE, FRAMES_PER_BUFFER, 0,
+				wave_samplerate, FRAMES_PER_BUFFER, 0,
 				//paClipOff | paDitherOff,
 				paNoFlag,
 				pa_callback, (void *)userdata);
@@ -376,12 +519,12 @@ static int wave_open_sound()
 				       paInt16,
 				       NULL,
 				       /* general parameters */
-				       SAMPLE_RATE, FRAMES_PER_BUFFER, 0,
+				       wave_samplerate, FRAMES_PER_BUFFER, 0,
 				       //paClipOff | paDitherOff,
 				       paNoFlag,
 				       pa_callback, (void *)userdata);
 // 	  err = Pa_OpenDefaultStream(&pa_stream,0,2,paInt16,
-// 				     SAMPLE_RATE,
+// 				     wave_samplerate,
 // 				     FRAMES_PER_BUFFER,
 // 				     N_WAV_BUF,pa_callback,(void *)userdata);
 	  SHOW("wave_open_sound > Pa_OpenDefaultStream(2): err=%d (%s)\n",err, Pa_GetErrorText(err));
@@ -395,7 +538,7 @@ static int wave_open_sound()
 			  &pa_stream,
 			  NULL, /* no input */
 			  &myOutputParameters,
-			  SAMPLE_RATE,
+			  wave_samplerate,
 			  framesPerBuffer,
 			  paNoFlag,
 			  //			  paClipOff | paDitherOff,
@@ -410,7 +553,7 @@ static int wave_open_sound()
 			      &pa_stream,
 			      NULL, /* no input */
 			      &myOutputParameters,
-			      SAMPLE_RATE,
+			      wave_samplerate,
 			      framesPerBuffer,
 			      paNoFlag,
 			      //			  paClipOff | paDitherOff,
@@ -427,14 +570,14 @@ static int wave_open_sound()
 			       &pa_stream,
 			       NULL, /* no input */
 			       &myOutputParameters,
-			       SAMPLE_RATE,
+			       wave_samplerate,
 			       framesPerBuffer,
 			       paNoFlag,
 			       //			       paClipOff | paDitherOff,
 			       pa_callback,
 			       (void *)userdata);
 
-	  //	  err = Pa_OpenDefaultStream(&pa_stream,0,2,paInt16,(double)SAMPLE_RATE,FRAMES_PER_BUFFER,pa_callback,(void *)userdata);
+	  //	  err = Pa_OpenDefaultStream(&pa_stream,0,2,paInt16,(double)wave_samplerate,FRAMES_PER_BUFFER,pa_callback,(void *)userdata);
 	}
       mInCallbackFinishedState = false;
 #endif
@@ -463,7 +606,7 @@ static void update_output_parameters(int selectedDevice, const PaDeviceInfo *dev
   if (deviceInfo)
     {
       double aLatency = deviceInfo->defaultLowOutputLatency;
-      double aCoeff = round(0.100 / aLatency);
+//      double aCoeff = round(0.100 / aLatency);
 //      myOutputParameters.suggestedLatency = aCoeff * aLatency;  // to avoid glitches ?
       myOutputParameters.suggestedLatency =  aLatency;          // for faster response ?
       SHOW("Device=%d, myOutputParameters.suggestedLatency=%f, aCoeff=%f\n",
@@ -526,7 +669,7 @@ static void select_device(const char* the_api)
 					defaultAlsaIndex = hostInfo->defaultOutputDevice;
 					const PaDeviceInfo *deviceInfo = Pa_GetDeviceInfo( defaultAlsaIndex );
 					update_output_parameters(defaultAlsaIndex, deviceInfo);
-					if (Pa_IsFormatSupported(NULL, &myOutputParameters, SAMPLE_RATE) == 0)
+					if (Pa_IsFormatSupported(NULL, &myOutputParameters, wave_samplerate) == 0)
 					{
 						SHOW( "select_device > ALSA (default), name=%s (#%d)\n", deviceInfo->name, defaultAlsaIndex);		  
 						selectedIndex = defaultAlsaIndex;
@@ -541,7 +684,7 @@ static void select_device(const char* the_api)
 	
 				update_output_parameters(i, deviceInfo);
 	
-				if (Pa_IsFormatSupported(NULL, &myOutputParameters, SAMPLE_RATE) == 0)
+				if (Pa_IsFormatSupported(NULL, &myOutputParameters, wave_samplerate) == 0)
 				{
 					SHOW( "select_device > ALSA, name=%s (#%d)\n", deviceInfo->name, i);
 	
@@ -613,12 +756,13 @@ void wave_set_callback_is_output_enabled(t_wave_callback* cb)
 //<wave_init
 
 // TBD: the arg could be "alsa", "oss",...
-void wave_init()
+int wave_init(int srate)
 {
   ENTER("wave_init");
   PaError err;
 
   pa_stream = NULL;
+	wave_samplerate = srate;
   mInCallbackFinishedState = false;
   init_buffer();
 
@@ -629,6 +773,7 @@ void wave_init()
     {
       SHOW_TIME("wave_init > Failed to initialise the PortAudio sound");
     }
+    return err == paNoError;
 }
 
 //>
@@ -1023,7 +1168,7 @@ int wave_get_remaining_time(uint32_t sample, uint32_t* time)
     {
       // TBD: take in account time suplied by portaudio V18 API
       a_time = sample - myReadPosition;
-      a_time = 0.5 + (a_time * 1000.0) / SAMPLE_RATE;
+      a_time = 0.5 + (a_time * 1000.0) / wave_samplerate;
     }
   else
     {
@@ -1050,7 +1195,7 @@ void *wave_test_get_write_buffer()
 // notdef USE_PORTAUDIO
 
 
-void wave_init() {}
+int wave_init(int srate) {return 1;}
 void* wave_open(const char* the_api) {return (void *)1;}
 size_t wave_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSize) {return theSize;}
 int wave_close(void* theHandler) {return 0;}
